package com.persagy.apm.diagnose.timerecord.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.diagnose.timerecord.model.vo.*;
import com.persagy.apm.diagnose.timerecord.model.dto.*;

import java.util.List;

/**
 * 监测指标计算时间(CalculateTimeRecord) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-10 00:13:10
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertCalculateTimeRecordTool {

    ConvertCalculateTimeRecordTool INSTANCE = Mappers.getMapper(ConvertCalculateTimeRecordTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param calculateTimeRecord do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-10 00:13:10
    */
    CalculateTimeRecordItemVO convert2ResponseItemDTO(CalculateTimeRecord calculateTimeRecord);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param calculateTimeRecord do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-10 00:13:10
    */
    CalculateTimeRecordListItemVO convert2ResponseListItemDTO(CalculateTimeRecord calculateTimeRecord);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param calculateTimeRecordList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-10 00:13:10
    */
    List<CalculateTimeRecordListItemVO> convert2List(List<CalculateTimeRecord> calculateTimeRecordList);
    
    /**
    * addDTO转换为do
    *
    * @param addCalculateTimeRecordDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-10 00:13:10
    */
    CalculateTimeRecord convertAddDto2Entity(AddCalculateTimeRecordDTO addCalculateTimeRecordDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param calculateTimeRecord 更新前的do对象
    * @param updateCalculateTimeRecordDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-10 00:13:10
    */
    CalculateTimeRecord convertUpdateDto2Entity(@MappingTarget CalculateTimeRecord calculateTimeRecord, UpdateCalculateTimeRecordDTO updateCalculateTimeRecordDTO);
}

