package com.persagy.apm.diagnose.utils;

import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpRequest;
import org.apache.http.NameValuePair;
import org.apache.http.NoHttpResponseException;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLException;
import javax.net.ssl.SSLHandshakeException;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.net.UnknownHostException;
import java.util.*;

/**
 * http工具
 */
public class HttpUtils {

    private static final Logger log = LoggerFactory.getLogger(HttpUtils.class);
    private static final CloseableHttpClient HTTP_CLIENT;
    private static final String UTF8 = "utf-8";

    static {
        //加载配置文件
        InputStream inputStream = HttpUtils.class.getResourceAsStream("/config/system.properties");
        Properties properties = new Properties();
        if (inputStream != null) {
            try {
                properties.load(inputStream);
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }

        String connectTimeoutString = properties.getProperty("httpclient.connect.timeout", "1000");
        String connectionRequestTimeoutString = properties.getProperty("httpclient.request.timeout", "1000");
        String socketTimeoutString = properties.getProperty("httpclient.socket.timeout", "3600000");

        String maxTotalString = properties.getProperty("httpclient.pool.total", "1200");
        String defaultMaxPerRouteString = properties.getProperty("httpclient.pool.per.route", "200");

        ConnectionSocketFactory plainsf = PlainConnectionSocketFactory.getSocketFactory();
        LayeredConnectionSocketFactory sslsf = SSLConnectionSocketFactory.getSocketFactory();
        Registry<ConnectionSocketFactory> registry = RegistryBuilder
                .<ConnectionSocketFactory>create().register("http", plainsf)
                .register("https", sslsf).build();
        PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager(registry);
        // 将最大连接数增加
        Integer maxTotal = Integer.valueOf(maxTotalString);
        cm.setMaxTotal(maxTotal);
        // 将每个路由基础的连接增加
        Integer defaultMaxPerRoute = Integer.valueOf(defaultMaxPerRouteString);
        cm.setDefaultMaxPerRoute(defaultMaxPerRoute);
        //host
        /*HttpHost httpHost = new HttpHost(hostname, port);
        // 将目标主机的最大连接数增加
        cm.setMaxPerRoute(new HttpRoute(httpHost), maxRoute);*/

        // 请求重试处理
        HttpRequestRetryHandler httpRequestRetryHandler = (exception, executionCount, context) -> {
            // 如果已经重试了5次，就放弃
            if (executionCount >= 5) {
                return false;
            }
            // 如果服务器丢掉了连接，那么就重试
            if (exception instanceof NoHttpResponseException) {
                return true;
            }
            // 不要重试SSL握手异常
            if (exception instanceof SSLHandshakeException) {
                return false;
            }
            // 连接被拒绝
            if (exception instanceof ConnectTimeoutException) {
                return false;
            }
            // 超时
            if (exception instanceof InterruptedIOException) {
                return false;
            }
            // 目标服务器不可达
            if (exception instanceof UnknownHostException) {
                return false;
            }
            // SSL握手异常
            if (exception instanceof SSLException) {
                return false;
            }

            HttpClientContext clientContext = HttpClientContext
                    .adapt(context);
            HttpRequest request = clientContext.getRequest();
            // 如果请求是幂等的，就再次尝试
            if (!(request instanceof HttpEntityEnclosingRequest)) {
                return true;
            }
            return false;
        };

        //requestConfig
        Integer connectionTimeout = Integer.valueOf(connectTimeoutString);
        Integer requestTimeout = Integer.valueOf(connectionRequestTimeoutString);
        Integer socketTimeout = Integer.valueOf(socketTimeoutString);
        RequestConfig requestConfig = RequestConfig.custom()
                // 连接建立时间，三次握手完成时间
                .setConnectTimeout(connectionTimeout)
                // 数据传输过程中数据包之间间隔的最大时间
                .setSocketTimeout(socketTimeout)
                // httpClient使用连接池来管理连接，这个时间就是从连接池获取连接的超时时间
                .setConnectionRequestTimeout(requestTimeout).build();
        HTTP_CLIENT = HttpClients.custom()
                .setConnectionManager(cm)
                .setRetryHandler(httpRequestRetryHandler)
                .setDefaultRequestConfig(requestConfig).build();
    }

    public static String httpGetRequest(String url) throws Exception {
        HttpGet httpGet = new HttpGet(url);
        return sendRequest(httpGet);
    }

    public static String httpPostRequest(String url) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        return sendRequest(httpPost);
    }

    public static String httpPostRequest(String url, Map<String, Object> params) throws IOException {
        log.info("请求url：" + url);
        log.info("参数是：{}", params.toString());
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
        httpPost.setEntity(new StringEntity(JsonUtils.beanToJson(params), UTF8));
        String respContent = sendRequest(httpPost);
        log.info("返回值是：{}", respContent);
        return respContent;
    }

    public static String httpPostFormUrlencoded(String url, Map<String, Object> params) throws IOException {
        log.info("请求url：" + url);
        log.info("参数是：{}", params.toString());
        HttpPost httpPost = new HttpPost(url);
        List<NameValuePair> pairs = new ArrayList<>();
        Map.Entry entry;
        for (Iterator var = params.entrySet().iterator(); var.hasNext(); pairs.add(new BasicNameValuePair((String) entry.getKey(), entry.getValue().toString()))) {
            entry = (Map.Entry) var.next();
        }
        httpPost.setEntity(new UrlEncodedFormEntity(pairs, UTF8));
        String respContent = sendRequest(httpPost);
        log.info("返回值是：{}", respContent);
        return respContent;
    }

    private static String sendRequest(HttpUriRequest request) throws IOException {
        CloseableHttpResponse response = HTTP_CLIENT.execute(request);
        return EntityUtils.toString(response.getEntity(), UTF8);
    }

    public static String httpPostRequest(String url, InputStream is) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(new InputStreamEntity(is));
        return sendRequest(httpPost);
    }

    public static String httpPostRequest(String url, File file) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(new FileEntity(file));
        return sendRequest(httpPost);
    }

    public static byte[] httpGetFile(String url) throws Exception {
        HttpGet httpGet = new HttpGet(url);
        CloseableHttpResponse response = HTTP_CLIENT.execute(httpGet);
        return EntityUtils.toByteArray(response.getEntity());
    }

}
